--
-- ReplaceVehicleTypes
--
-- @author:    	Xentro (Marcus@Xentro.se)
-- @website:	www.Xentro.se
-- @history:	2016-12-18 - Initial implementation
--				2018-03-13 - Update
-- 

ReplaceVehicleTypes = {};

ReplaceVehicleTypes.modName 			= g_currentModName;
ReplaceVehicleTypes.modDirectory 		= g_currentModDirectory;

-- Table row names for Utils.vehicleTypesToReplace
-- 1 = XML filename
-- 2 = New vehicle type
-- 3 = Mod name of target
-- 4 = Mod name of this zip

addModEventListener(ReplaceVehicleTypes);

function ReplaceVehicleTypes:loadMap(name)
	local xmlFile = loadXMLFile("tempModDesc", Utils.getFilename("modDesc.xml", ReplaceVehicleTypes.modDirectory));
	
	local i = 0;
	while true do
		local key = string.format("modDesc.changeVehicleTypes.type(%d)", i);
		if not hasXMLProperty(xmlFile, key) then break; end;
		
		local xmlFilename = getXMLString(xmlFile, key .. "#xmlFilename");
		local newVehicleType = getXMLString(xmlFile, key .. "#newVehicleType");
		
		if xmlFilename ~= nil and xmlFilename ~= "" and newVehicleType ~= nil and newVehicleType ~= "" then
			table.insert(Utils.vehicleTypesToReplace, {xmlFilename:lower(), newVehicleType, {}, ReplaceVehicleTypes.modName});
		end;
		
		i = i + 1;
	end;
	
	delete(xmlFile);
end;

function ReplaceVehicleTypes:deleteMap()
end;

function ReplaceVehicleTypes:mouseEvent(posX, posY, isDown, isUp, button)
end;

function ReplaceVehicleTypes:keyEvent(unicode, sym, modifier, isDown)
end;

function ReplaceVehicleTypes:update(dt)
end;

function ReplaceVehicleTypes:draw()
end;


-- Override the global loadXMLFile and update for our need. vehicleType is loaded before we can set it..
function ReplaceVehicleTypes:masterOverride()
	function callVehicleTypeUpdate(self, oldFunc, ...)
		oldFunc(self, ...);
		
		-- Search for xml file name and add mod name to list
		for i, v in ipairs(StoreItemsUtil.storeItems) do
			if v.customEnvironment ~= nil then	-- only mods
				for i, p in ipairs(Utils.vehicleTypesToReplace) do
					if v.xmlFilenameLower:find(p[1]) ~= nil then
						local modName, baseDirectory = Utils.getModNameAndBaseDirectory(v.xmlFilename);
						
						-- If we find more mods with the same xml name then this will/should support that.
						table.insert(p[3], modName);
						
						-- print("-- Searching -- Target vehicle (P3):" .. modName .. ", New vehicleType (P2): " .. p[2] .. ", xmlFile (P1): " .. p[1]);
						
						-- Print the table row for store item
						-- DebugUtil.printTableRecursively(v, i .. " -- ", 0, 1);
						
						break;
					end;
				end;
			end;
		end;
		
		-- We must do this part after all the register scripts!
		-- Process list from above and create new vehicleType
		for _, p in ipairs(Utils.vehicleTypesToReplace) do
			for _, n in ipairs(p[3]) do
				local newName = n .. "." .. p[2];
				local vt = VehicleTypeUtil.vehicleTypes[p[4] .. "." .. p[2]];
				
				if vt ~= nil then
					VehicleTypeUtil.vehicleTypes[newName] = {
						name = vt.name,
						className = vt.className,
						filename = vt.filename,
						specializations = vt.specializations
					};
					
					-- print("-- Processing -- Register vehicle type (P3.P2): " .. newName .. ", Name of this mod zip (P4): " .. p[4] .. ", file (P1): " .. p[1]);
				else
					print("ReplaceVehicleTypes - ERROR: vehicleType '" .. p[4] .. "." .. p[2] .."' do not exist.");
				end;
			end;
		end;
	end;
	BaseMission.loadMapFinished = Utils.overwrittenFunction(BaseMission.loadMapFinished, callVehicleTypeUpdate);

	
	local oldFunc = getfenv(0)["loadXMLFile"];
	function loadXMLFileUpdated(name, file)
		local xmlFile = oldFunc(name, file);
		
		-- This is called 2 times, we only need the 1th but we do both, better be safe then conflicts.
		if name == "TempConfig" then
			if hasXMLProperty(xmlFile, "vehicle#type") then
				for _, p in ipairs(Utils.vehicleTypesToReplace) do
					file = file:lower();
					
					if file:find(p[1]) ~= nil then
						-- debug print only once.
						if not p[5] then
							-- print("-- Finished -- Setting the new type (" .. p[2] .. ") for Target mod " .. p[3][1] .. " - " .. file);
							p[5] = true;
						end;
						
						setXMLString(xmlFile, "vehicle#type", p[2]);
						
						break;
					end;
				end;
			end;
		end;
		
		return xmlFile;
	end;
	
	getfenv(0)["loadXMLFile"] = loadXMLFileUpdated;
end;



-- An approuch to let other modders use the same approuch as me without needing to override the same function more then once
-- Be sure to keep the base function the same as even small changes could do damage to the script
local masterVersion = 1.0;
if (g_masterOverrides == nil or g_masterOverrides ~= nil and g_masterOverrides[1] < masterVersion) then
	-- print("-- Master -- " .. g_currentModName .. " is being set as master, version " .. masterVersion);
	-- Table row names for g_masterOverrides
	-- 1 = Version
	-- 2 = Functions to be called
	-- 3 = Backup function of BaseMission.loadMap
	-- 4 = Master have been called
	
	if g_masterOverrides == nil then
		-- Make table global accessible
		getfenv(0)["g_masterOverrides"] = {};
		g_masterOverrides[2] = {};
		g_masterOverrides[3] = BaseMission.loadMap;
	else
		-- Reseting the override of loadMap function would slim down our footprint but at the cost of mod conflicts if other try to override the same function.
		BaseMission.loadMap = g_masterOverrides[3];
	end;
	
	g_masterOverrides[1] = masterVersion;
	
	function modsAreFinished(self, oldFunc, ...)
		-- We have finished the loading of mods, activate overrides
		if g_masterOverrides ~= nil and not g_masterOverrides[4] then
			-- print("-- Master -- Calling master overrides");
			g_masterOverrides[4] = true;
			
			for _, f in pairs(g_masterOverrides[2]) do
				f();
			end;
		end;
		
		return oldFunc(self, ...);
	end;
	
	BaseMission.loadMap = Utils.overwrittenFunction(BaseMission.loadMap, modsAreFinished);
end;
-- Ends here


-- We only want 1 master override to be called.
local replaceVersion = 1;
if Utils.vehicleTypesToReplace == nil then
	Utils.vehicleTypesToReplace = {};

	-- Add our master override for ReplaceVehicleTypes
	table.insert(g_masterOverrides[2], ReplaceVehicleTypes.masterOverride);
	
	-- 1 = Version
	-- 2 = Class table
	-- 3 = Table index in master table
	
	Utils.vehicleTypesToReplaceMaster = {};
	Utils.vehicleTypesToReplaceMaster[1] = replaceVersion;
	Utils.vehicleTypesToReplaceMaster[2] = ReplaceVehicleTypes;
	Utils.vehicleTypesToReplaceMaster[3] = table.getn(g_masterOverrides[2]);
else
	-- Replace old data with this new as this mod has an newer version.
	if Utils.vehicleTypesToReplaceMaster[1] < replaceVersion then
		Utils.vehicleTypesToReplaceMaster[1] = replaceVersion;
		Utils.vehicleTypesToReplaceMaster[2] = ReplaceVehicleTypes;
		
		-- Remove old and add new master function for ReplaceVehicleTypes
		table.remove(g_masterOverrides[2], Utils.vehicleTypesToReplaceMaster[3]);
		table.insert(g_masterOverrides[2], ReplaceVehicleTypes.masterOverride);
		
		-- Get new table index
		Utils.vehicleTypesToReplaceMaster[3] = table.getn(g_masterOverrides[2]);
	end;
end;